import { FastifyReply, FastifyRequest } from 'fastify';
import * as node_fetch from 'node-fetch';
import * as fastify_types_route from 'fastify/types/route';
import * as http from 'http';

declare type Snowflake = `${bigint}`;

/**
 * These type definitions come from the official Discord API docs. They should
 * be defined with references back to the documentation section.
 */
declare type GuildMember = Record<string, unknown>;

/**
 * These type definitions come from the official Discord API docs. They should
 * be defined with references back to the documentation section.
 */
declare type User = Record<string, unknown>;

/**
 * These type definitions come from the official Discord API docs. They should
 * be defined with references back to the documentation section.
 */

declare type Channel = Record<string, unknown>;
declare enum AllowedMentionTypes {
    ROLE = "roles",
    USER = "users",
    EVERYONE = "everyone"
}
declare type AllowedMentions = {
    parse: AllowedMentionTypes[];
    roles: Snowflake[];
    users: Snowflake[];
    replied_user: boolean;
};

/**
 * These type definitions come from the official Discord API docs. They should
 * be defined with references back to the documentation section.
 */
declare type Embed$1 = {
    title?: string;
    type?: EmbedType;
    description?: string;
    url?: string;
    timestamp?: Date;
    color?: number;
    footer?: EmbedFooter;
    image?: EmbedImage;
    thumbnail?: EmbedThumbnail;
    video?: EmbedVideo;
    provider?: EmbedProvider;
    author?: EmbedAuthor;
    fields?: EmbedField[];
};
declare enum EmbedType {
    RICH = 0,
    IMAGE = 1,
    VIDEO = 2,
    GIFV = 3,
    ARTICLE = 4,
    LINK = 5
}
declare type EmbedThumbnail = {
    url?: string;
    proxy_url?: string;
    height?: number;
    width?: number;
};
declare type EmbedVideo = {
    url?: string;
    proxy_url?: string;
    height?: number;
    width?: number;
};
declare type EmbedImage = {
    url?: string;
    proxy_url?: string;
    height?: number;
    width?: number;
};
declare type EmbedProvider = {
    name?: string;
    url?: string;
};
declare type EmbedAuthor = {
    name?: string;
    url?: string;
    icon_url?: string;
    proxy_icon_url?: string;
};
declare type EmbedFooter = {
    text: string;
    icon_url?: string;
    proxy_icon_url?: string;
};
declare type EmbedField = {
    name: string;
    value: string;
    inline?: boolean;
};

/**
 * These type definitions come from the official Discord API docs. They should
 * be defined with references back to the documentation section.
 */
declare type Role = Record<string, unknown>;

/**
 * These type definitions come from the official Discord API docs. They should
 * be defined with references back to the documentation section.
 */

declare type Emoji = {
    id: Snowflake | null;
    name: string | null;
    roles?: Snowflake[];
    user?: User;
    require_colons?: boolean;
    managed?: boolean;
    animated?: boolean;
    available?: boolean;
};

/**
 * These type definitions come from the official Discord API docs. They should
 * be defined with references back to the documentation section.
 */

declare type Component = {
    type: ComponentType;
    style?: ButtonStyle;
    label?: string;
    emoji?: Pick<Emoji, "id" | "name" | "animated">;
    custom_id?: string;
    url?: string;
    disabled?: boolean;
    components?: Component[];
};
declare enum ComponentType {
    ACTION_ROW = 1,
    BUTTON = 2,
    SELECT = 3
}
declare enum ButtonStyle {
    PRIMARY = 1,
    SECONDARY = 2,
    SUCCESS = 3,
    DANGER = 4,
    LINK = 5
}

declare type Message = Record<string, unknown>;

/**
 * These type definitions come from the official Discord API docs. They should
 * be defined with references back to the documentation section.
 */

declare type ApplicationCommand = {
    id: Snowflake;
    application_id: Snowflake;
    name: string;
    description: string;
    options?: ApplicationCommandOption[];
    default_permission?: boolean;
};
declare type ApplicationCommandOption = {
    type: ApplicationCommandOptionType;
    name: string;
    description: string;
    required?: boolean;
    choices?: ApplicationCommandOptionChoice[];
    options?: ApplicationCommandOption[];
};
declare enum ApplicationCommandOptionType {
    SUB_COMMAND = 1,
    SUB_COMMAND_GROUP = 2,
    STRING = 3,
    INTEGER = 4,
    BOOLEAN = 5,
    USER = 6,
    CHANNEL = 7,
    ROLE = 8,
    MENTIONABLE = 9
}
declare type ApplicationCommandOptionChoice = {
    name: string;
    value: string | number;
};
declare type Interaction$1 = {
    id: Snowflake;
    application_id: Snowflake;
    type: InteractionRequestType;
    data?: ApplicationCommandInteractionData;
    guild_id?: Snowflake;
    channel_id?: Snowflake;
    member?: GuildMember;
    user?: User;
    token: string;
    version: number;
    message?: Message;
};
declare enum InteractionRequestType {
    PING = 1,
    APPLICATION_COMMAND = 2,
    MESSAGE_COMPONENT = 3
}
declare type ApplicationCommandInteractionData = {
    id: Snowflake;
    name: string;
    resolved?: ApplicationCommandInteractionDataResolved;
    options?: ApplicationCommandInteractionDataOption[];
    custom_id?: string;
    component_type?: ComponentType;
};
declare type ApplicationCommandInteractionDataResolved = {
    users?: Record<Snowflake, User>;
    members?: Record<Snowflake, Omit<GuildMember, "user" | "deaf" | "mute">>;
    roles?: Record<Snowflake, Role>;
    channels?: Record<Snowflake, Pick<Channel, "id" | "name" | "type" | "permissions">>;
};
declare type ApplicationCommandInteractionDataOption = {
    name: string;
    type: ApplicationCommandOptionType;
    value?: OptionType;
    options?: ApplicationCommandInteractionDataOption[];
};
declare type OptionType = unknown;
declare type InteractionApplicationCommandCallbackData = {
    tts?: boolean;
    content?: string;
    embeds?: Embed$1[];
    allowed_mentions?: AllowedMentions;
    flags?: number;
    components?: Component[];
};

/**
 * Snowflake structure is defined here:
 * https://discord.com/developers/docs/reference#snowflakes-snowflake-id-format-structure-left-to-right
 */
declare function getTimestamp(snowflake: Snowflake): Date;
declare function getWorkerID(snowflake: Snowflake): number;
declare function getProcessID(snowflake: Snowflake): number;
declare function getIncrement(snowflake: Snowflake): number;

declare const snowflake_getTimestamp: typeof getTimestamp;
declare const snowflake_getWorkerID: typeof getWorkerID;
declare const snowflake_getProcessID: typeof getProcessID;
declare const snowflake_getIncrement: typeof getIncrement;
declare namespace snowflake {
  export {
    snowflake_getTimestamp as getTimestamp,
    snowflake_getWorkerID as getWorkerID,
    snowflake_getProcessID as getProcessID,
    snowflake_getIncrement as getIncrement,
  };
}

declare function getGlobalApplicationCommands(): Promise<void>;
declare function getGuildApplicationCommands(guildID: Snowflake, options?: {
    headers?: Record<string, string>;
    applicationID?: Snowflake;
}): Promise<ApplicationCommand[]>;
declare function postGuildApplicationCommand(guildID: Snowflake, command: Omit<ApplicationCommand, "id" | "application_id">, options?: {
    headers?: Record<string, string>;
    applicationID?: Snowflake;
}): Promise<ApplicationCommand>;
declare function putGuildApplicationCommands(guildID: Snowflake, commands: ApplicationCommand[], options?: {
    headers?: Record<string, string>;
    applicationID?: Snowflake;
}): Promise<ApplicationCommand>;
declare function patchGuildApplicationCommand(guildID: Snowflake, command: Optional<ApplicationCommand, "name" | "description" | "options" | "default_permission">, options?: {
    headers?: Record<string, string>;
    applicationID?: Snowflake;
}): Promise<ApplicationCommand>;
declare function deleteGuildApplicationCommand(guildID: Snowflake, commandID: Snowflake, options?: {
    headers?: Record<string, string>;
    applicationID?: Snowflake;
}): Promise<boolean>;

declare function getGuild(id: Snowflake, options?: {
    headers?: Record<string, string>;
    counts?: boolean;
}): Promise<node_fetch.Response>;

/**
 * https://discord.com/developers/docs/interactions/slash-commands#followup-messages
 */
declare function postWebhookMessage(interactionToken: string, data: InteractionApplicationCommandCallbackData, options?: {
    headers?: Record<string, string>;
    applicationID?: Snowflake;
}): Promise<node_fetch.Response>;
declare function patchWebhookMessage(interactionToken: string, id: string, data: InteractionApplicationCommandCallbackData, options?: {
    headers?: Record<string, string>;
    applicationID?: string;
}): Promise<node_fetch.Response>;
declare function deleteWebhookMessage(interactionToken: string, id: string, options?: {
    headers?: Record<string, string>;
    applicationID?: string;
}): Promise<boolean>;

declare const index_getGlobalApplicationCommands: typeof getGlobalApplicationCommands;
declare const index_getGuildApplicationCommands: typeof getGuildApplicationCommands;
declare const index_postGuildApplicationCommand: typeof postGuildApplicationCommand;
declare const index_putGuildApplicationCommands: typeof putGuildApplicationCommands;
declare const index_patchGuildApplicationCommand: typeof patchGuildApplicationCommand;
declare const index_deleteGuildApplicationCommand: typeof deleteGuildApplicationCommand;
declare const index_getGuild: typeof getGuild;
declare const index_postWebhookMessage: typeof postWebhookMessage;
declare const index_patchWebhookMessage: typeof patchWebhookMessage;
declare const index_deleteWebhookMessage: typeof deleteWebhookMessage;
declare namespace index {
  export {
    index_getGlobalApplicationCommands as getGlobalApplicationCommands,
    index_getGuildApplicationCommands as getGuildApplicationCommands,
    index_postGuildApplicationCommand as postGuildApplicationCommand,
    index_putGuildApplicationCommands as putGuildApplicationCommands,
    index_patchGuildApplicationCommand as patchGuildApplicationCommand,
    index_deleteGuildApplicationCommand as deleteGuildApplicationCommand,
    index_getGuild as getGuild,
    index_postWebhookMessage as postWebhookMessage,
    index_patchWebhookMessage as patchWebhookMessage,
    index_deleteWebhookMessage as deleteWebhookMessage,
  };
}

declare class Embed implements Serializable {
    title: Embed$1["title"];
    description: Embed$1["description"];
    url: Embed$1["url"];
    timestamp: Embed$1["timestamp"];
    color: Embed$1["color"];
    footer: EmbedFooter | undefined;
    image: EmbedImage | undefined;
    thumbnail: EmbedThumbnail | undefined;
    video: EmbedVideo | undefined;
    provider: EmbedProvider | undefined;
    author: EmbedAuthor | undefined;
    fields: EmbedField[];
    constructor(options: Embed$1 | null);
    setTitle(title: Embed$1["title"]): this;
    setDescription(description: Embed$1["description"]): this;
    setURL(url: Embed$1["url"]): this;
    setTimestamp(timestamp: Embed$1["timestamp"]): this;
    setColor(color: Embed$1["color"]): this;
    setFooter(footer: EmbedFooter): this;
    setImage(image: EmbedImage): this;
    setThumbnail(thumbnail: EmbedThumbnail): this;
    setProvider(provider: EmbedProvider): this;
    setAuthor(author: EmbedAuthor): this;
    addField(field: EmbedField): this;
    serialize(): Embed$1;
}

declare type Optional<T, K extends keyof T> = Omit<T, K> & Partial<Pick<T, K>>;
interface Serializable {
    serialize(): unknown;
}
interface SerializableComponent extends Serializable {
    get id(): Component["custom_id"];
    get type(): ComponentType;
    serialize(): Component;
}
declare type InteractionReply = {
    message?: string;
    embed?: Embed | Embed[] | null;
    components?: SerializableComponent[] | null;
    ephemeral?: boolean;
    queue?: boolean;
};
declare type InteractionMessageModifiers = {
    edit: (data: InteractionApplicationCommandCallbackData, id?: string) => ReturnType<typeof patchWebhookMessage>;
    delete: (id?: string) => ReturnType<typeof deleteWebhookMessage>;
};
interface Interaction {
    readonly type: InteractionRequestType;
    readonly messages: InteractionMessageModifiers;
    acknowledge: () => unknown;
    reply: (message: InteractionReply) => unknown;
}

declare type ChoiceValue = ApplicationCommandOptionChoice["value"];
declare type InputArgs = {
    type: ApplicationCommandOptionType;
    name: string;
    description: string;
    required?: boolean;
    choices?: Choices<ChoiceValue>;
    options?: ApplicationCommandOption[];
};
declare class Input implements Serializable {
    readonly type: ApplicationCommandOptionType;
    readonly name: string;
    readonly description: string;
    readonly required: boolean;
    readonly options: ApplicationCommandOption[] | undefined;
    readonly choices: Choices<string | number> | undefined;
    constructor({ type, name, description, choices, options, required, }: InputArgs);
    serialize(): ApplicationCommandOption;
}
interface StringInputArgs extends Omit<InputArgs, "type" | "options"> {
    choices: Choices<string>;
}
declare class StringInput extends Input {
    constructor(args: StringInputArgs);
}
interface IntegerInputArgs extends Omit<InputArgs, "type" | "options"> {
    choices: Choices<number>;
}
declare class IntegerInput extends Input {
    constructor(args: IntegerInputArgs);
}
declare class BooleanInput extends Input {
    constructor(args: Omit<InputArgs, "type" | "choices" | "options">);
}
declare class UserInput extends Input {
    constructor(args: Omit<InputArgs, "type" | "choices" | "options">);
}
declare class ChannelInput extends Input {
    constructor(args: Omit<InputArgs, "type" | "choices" | "options">);
}
declare class RoleInput extends Input {
    constructor(args: Omit<InputArgs, "type" | "choices" | "options">);
}
declare class MentionableInput extends Input {
    constructor(args: Omit<InputArgs, "type" | "choices" | "options">);
}
declare type ExplicitChoice<T extends ChoiceValue> = {
    name: string;
    value: T;
};
declare class Choices<T extends ChoiceValue> implements Serializable {
    readonly _choices: Map<string, ApplicationCommandOptionChoice>;
    constructor(choices: Record<string, T | ExplicitChoice<T>>);
    static list<T extends ChoiceValue, U extends Record<string, T | ExplicitChoice<T>>>(this: typeof Choices, choices: U): Choices<T> & U;
    serialize(): ApplicationCommandOptionChoice[];
}

declare class ApplicationCommandInteraction implements Interaction {
    #private;
    readonly type = InteractionRequestType.APPLICATION_COMMAND;
    readonly name: string;
    readonly token: string;
    readonly response: FastifyReply;
    readonly messages: InteractionMessageModifiers;
    readonly channelID: Snowflake | undefined;
    readonly guildID: Snowflake | undefined;
    readonly member: Record<string, unknown> | undefined;
    constructor(application: Application, request: FastifyRequest<{
        Body: Interaction$1;
    }>, response: FastifyReply);
    get options(): {};
    acknowledge(): FastifyReply<http.Server, http.IncomingMessage, http.ServerResponse, fastify_types_route.RouteGenericInterface, unknown>;
    reply({ message, embed, components, ephemeral, queue, }: InteractionReply): Promise<node_fetch.Response | "@original">;
}

declare type CommandArgs = {
    name: string;
    description: string;
    defaultPermission?: boolean;
    options?: Input[];
    handler: (interaction: ApplicationCommandInteraction) => unknown;
};
declare class Command implements Serializable {
    #private;
    name: string;
    handler: (interaction: ApplicationCommandInteraction, application: Application) => unknown;
    constructor({ name, description, options, handler, defaultPermission, }: CommandArgs);
    group(): void;
    subcommand(): void;
    /**
     * Compares if a command is equal to a given command.
     * @param schema
     */
    equals(schema: ApplicationCommand): boolean;
    serialize(): Optional<ApplicationCommand, "id" | "application_id">;
}

declare type ApplicationArgs = {
    applicationID: string;
    publicKey: string;
    token: string;
    port?: number;
};
declare class Application {
    #private;
    constructor({ applicationID, publicKey, token, port }: ApplicationArgs);
    get id(): `${bigint}`;
    addCommand(command: Command): this;
    addCommands(...commands: Command[]): this;
    addComponent(component: SerializableComponent): void;
    updateCommands(): Promise<this>;
    handler(request: FastifyRequest<{
        Body: Interaction$1;
    }>, response: FastifyReply): unknown;
    startServer(): void;
}

declare class PingInteraction {
    readonly type = InteractionRequestType.PING;
}

declare class MessageComponentInteraction implements Interaction {
    #private;
    readonly type = InteractionRequestType.MESSAGE_COMPONENT;
    readonly token: string;
    readonly response: FastifyReply;
    readonly customID: string;
    readonly messages: InteractionMessageModifiers;
    readonly channelID: Snowflake | undefined;
    readonly guildID: Snowflake | undefined;
    readonly member: Record<string, unknown> | undefined;
    readonly message: Record<string, unknown> | undefined;
    constructor(application: Application, request: FastifyRequest<{
        Body: Interaction$1;
    }>, response: FastifyReply);
    acknowledge(): FastifyReply<http.Server, http.IncomingMessage, http.ServerResponse, fastify_types_route.RouteGenericInterface, unknown>;
    reply({ message, embed, components, ephemeral, queue, }: InteractionReply): Promise<node_fetch.Response | "@original">;
}

declare class ActionRow implements SerializableComponent {
    components: SerializableComponent[];
    constructor(...components: SerializableComponent[]);
    get id(): string;
    get type(): ComponentType;
    get isFull(): boolean;
    setComponents(...components: SerializableComponent[]): void;
    addComponent(component: SerializableComponent): void;
    serialize(): Component;
}

declare type ButtonArgs = {
    handler: (event: MessageComponentInteraction, application: Application) => unknown;
    customID: Component["custom_id"];
} & Omit<Component, "type" | "url" | "custom_id" | "components">;
declare type ButtonLinkArgs = Omit<Component, "type" | "custom_id" | "style" | "components">;
declare class Button implements SerializableComponent {
    #private;
    handler: ButtonArgs["handler"] | undefined;
    constructor(options: ButtonArgs | ButtonLinkArgs);
    get id(): string | undefined;
    get type(): ComponentType;
    setLabel(label: ButtonArgs["label"]): this;
    setEmoji(emoji: ButtonArgs["emoji"]): this;
    setCustomID(customID: ButtonArgs["customID"]): this;
    setStyle(style: ButtonArgs["style"]): this;
    setHandler(fn: ButtonArgs["handler"]): this;
    setURL(url: ButtonLinkArgs["url"]): this;
    setDisabled(disabled: ButtonArgs["disabled"]): this;
    serialize(): Component;
}

export { index as API, ActionRow, Application, ApplicationCommandInteraction, BooleanInput, Button, ButtonStyle, ChannelInput, Choices, Command, Embed, Input, IntegerInput, MentionableInput, MessageComponentInteraction, PingInteraction, RoleInput, snowflake as SnowflakeUtils, StringInput, UserInput };
